/* * * * * * * * * * * *\
 *    Panel-Docklet    *
 *   version: Friday   *
 * Jodli  (pw3@gmx.at) *
\* * * * * * * * * * * */

/*\
 * IDEAS:
 * 
 * (possible?) fill() - ignoriert minimized (Abfrage dafür suchen)
 * (possible?) window-preferences (arbeitsflaeche wechseln, vordergrund, ...)
 * show-timeout
 * bottom-height mit strut
\*/

const St = imports.gi.St;
const Main = imports.ui.main;
const LayoutManager = Main.layoutManager;
const Shell = imports.gi.Shell;
const Lang = imports.lang;
const Tweener = imports.ui.tweener;
const DND = imports.ui.dnd;
const Mainloop = imports.mainloop;
const Gio = imports.gi.Gio;
const AppFavorites = imports.ui.appFavorites;
const AppDisplay = imports.ui.appDisplay;
const PopupMenu = imports.ui.popupMenu;
const ModalDialog = imports.ui.modalDialog;
const Clutter = imports.gi.Clutter;

const Extention_path = imports.ui.extensionSystem.extensionMeta["panel-docklet@quina.at"].path;
const Gettext = imports.gettext;
const _ = Gettext.gettext;

const MINIMIZE_WINDOW=0;
const NEW_WINDOW=1;
const CLOSE_WINDOW=2;

const LEFT = 0;
const MIDDLE = 1;
const RIGHT = 2;

const TOP = 0;
const BOTTOM = 1;

const DOCK_PADDING_BOTTOM = 3;
const DOCK_PADDING_LEFT = 3;
const DOCK_PADDING_RIGHT = 3;
const DOCK_WS_LABEL_FONT_SIZE = 10;
const WS_LABEL_WIDTH = 20;
const WS_LINES_SPACE_BETWEEN = 3;
const WS_LINES_SPACE_LEFT = 4;
const ICON_PADDING_TOP = 3;
const ICON_PADDING_SUM = 6;
const ICON_MINIMIZED_OPACITY = 80;
const ICON_COPY_OPACITY = 50;
const ICON_LABEL_SPACE = 2;
const BUTTON_MAX_WIDTH = 200;
const IF_STRUT_AND_TOP_Y = 21;
const MOUSE_POLL_FREQUENCY = 50;

let mainPanelDocklet, secondaryPanelDocklets = [], ID_monitorEvent;


function SettingsContainer(Extention_path, id) {
	this._init(Extention_path, id);
}
SettingsContainer.prototype = {
	_init: function(Extention_path, id) {
		this._connector = {};
		
		this._conf = {};
		
		//in theory, id could be a great security leak. But extentions already have file-system-rights anyway...
		this._file = Gio.file_new_for_path(Extention_path + "/settings"+id+".json");
		
		if(this._file.query_exists(null)) {
			[flag, data] = this._file.load_contents(null);
			
			if(flag)
				this._conf = JSON.parse(data);
			else {
				global.log("Panel-Docklet: Something's wrong... I wasn't able to load the settings... I'll ignore that and get you the default-settings instead.");
				this.restoreDefault(Extention_path);
			}
			//no error: We want to be able to save it anyway
			this._error = false;
		}
		else {
			global.log("Panel-Docklet: Uh, there are no settings saved for that Box. I'll get you the default settings instead.");
			this.restoreDefault(Extention_path);
		}
	},
	
	get_boolean: function(k) {
		return this._conf[k] || false;
	},
	get_double: function(k) {
		return this._conf[k] || 0;
	},
	get_int: function(k) {
		return parseInt(this._conf[k]);
	},
	get_enum: function(k) {
		return this._conf[k] || 0;
	},
	
	set_boolean: function(k, v, noEmit) {
		this._conf[k] = v;
		this._save_data(k, noEmit);
	},
	set_double: function(k, v, noEmit) {
		this._conf[k] = v;
		this._save_data(k, noEmit);
	},
	set_int: function(k, v, noEmit) {
		this._conf[k] = v;
		this._save_data(k, noEmit);
	},
	set_enum: function(k, v, noEmit) {
		this._conf[k] = v;
		this._save_data(k, noEmit);
	},
	
	restoreDefault: function(Extention_path) {
		this._conf = {};
		
		let file = Gio.file_new_for_path(Extention_path + "/default.json");
		if(file.query_exists(null)) {
			[flag, data] = file.load_contents(null);
			if(flag) {
				this._conf = JSON.parse(data);
				this._error = false;
			}
			else {
				global.log("Panel-Docklet: Something's terribly wrong! I wasn't able to load the default settings... I won't save anything in this session. And don't blame me, if Panel-Docklet is acting strangely...");
				this._error = true;
			}
		}
		else {
			global.log("Panel-Docklet: Something's terribly wrong! Neither your settings nor the default settings seem to exist... I won't save anything in this session. And don't blame me, if Panel-Docklet is acting strangely...");
			this._error = true;
		}
		this._save_data();
	},
	_save_data: function(k, noEmit) {
		if(!this._error)
			this._file.replace_contents(JSON.stringify(this._conf), null, false, 0, null);
		else
			global.log("Panel-Docklet: I really want to save that. But there was an error before...");
		
		if(!noEmit)
			this.emit(k);
	},
	
	connect: function(k, f) {
		this._connector[k] = f;
	},
	disconnect: function(k) {
		delete this._connector[k];
	},
	emit: function(k) {
		if(this._connector[k])
			this._connector[k](k, this._conf[k]);
	}
}

function panelDocklet(id, secondaryPanelDocklets) {
	this._init(id, secondaryPanelDocklets);
}
panelDocklet.prototype = {
	_init: function(id, secondaryPanelDocklets) {
		this._monitorId = id;
		this._secondaryPanelDocklets = secondaryPanelDocklets;
		this._winVarKey_icon = "PANEL_DOCKLET_icon"+id;
		this._winVarKey_addedToAll = "PANEL_DOCKLET_addedToAllWS"+id;
		
		let settings =	new SettingsContainer(Extention_path, id),
			settingsListener = [];
		
		this.settings = settings;
		let style_class = this._loadConfigs();
		
		
		this.actor = new St.BoxLayout({ name: "panelDocklet",
				reactive: true,
				x: this._calcDockX(this._dockRealWidth),
				y: this._dockY,
				width: this._dockRealWidth,
				height: this._dockHeight + DOCK_PADDING_BOTTOM,
				style_class: style_class});
		
		LayoutManager.addChrome(this.actor);
		
		if(this._CONF_cutStruts)
			this._addStrut();
		
		this._saveHotCorners();
		
		this._dragPointer = new St.Bin({style_class: "moveTo", opacity:0});
		this.actor.add(this._dragPointer);
		
		this._winLenRecord = 0;
		this._winLenRecordClass = null;
		
		this._box = new workspaceBox(this);
		this._box._indexAllWS();
		
		this._isHidden = false;
		
		if(this._CONF_autohide)
			this._hideDock();
		else if(this._CONF_showAllWS)
			this._box.listWS();
		
		this._menuIsOpened = false;
		this._ID_menuTimeout = false;
		this._menuManager = new PopupMenu.PopupMenuManager(this);
		
		this._lastZoomWS = null;
		this._ID_mouseTracking = false;
		
		if(this._CONF_autohide) {
			this._ID_enterEvent = this.actor.connect("enter-event", Lang.bind(this,this._showDock));
			this._ID_leaveEvent = this.actor.connect("leave-event", Lang.bind(this, this._hideDock));
		}
		else if(this._CONF_zoomEffect) {
			this._ID_enterEvent = this.actor.connect("enter-event", Lang.bind(this, this.startZooming));
			this._ID_leaveEvent = this.actor.connect("leave-event", Lang.bind(this, this.stopZooming));
		}
		
		this._ID_clickEvent = this.actor.connect("button-release-event", Lang.bind(this, this._onMouseClick));
		
		//only needed for real connects...
		//this._ID_settingsListener = settingsListener;
		//settingsListener.push( ... );
		settings.connect("zoom-effect", Lang.bind(this, this._reload));
		settings.connect("autohide", Lang.bind(this, this._update));
		settings.connect("hide-time", Lang.bind(this, this._loadConfigs));
		settings.connect("show-time", Lang.bind(this, this._loadConfigs));
		settings.connect("time-until-hide", Lang.bind(this, this._loadConfigs));
		settings.connect("middle-click-action", Lang.bind(this, this._loadConfigs));
		settings.connect("basic-icon-size", Lang.bind(this, this._reload));
		settings.connect("basic-button-width", Lang.bind(this, this._update));
		settings.connect("dock-width-percent", Lang.bind(this, this._update));
		settings.connect("dock-x-percent", Lang.bind(this, this._update));
		settings.connect("dock-y", Lang.bind(this, this._update));
		settings.connect("list-all-workspaces", Lang.bind(this, this._reload));
		settings.connect("show-workspace-number", Lang.bind(this, this._reload));
		settings.connect("dont-hide-size", Lang.bind(this, this._update));
		settings.connect("expand-box", Lang.bind(this, this._reload));
		settings.connect("fix-docklet-position-at", Lang.bind(this, this._update));
		settings.connect("show-window-title", Lang.bind(this, this._update));
		settings.connect("cut-view", Lang.bind(this, this._update));
		settings.connect("only-screen-windows", Lang.bind(this, this._reload));
	},
	_getMonitor: function() {
		let i = this._monitorId;
		return ((i=="primary") ? LayoutManager.primaryMonitor : LayoutManager.monitors[i])
	},
	_loadConfigs: function() {
		//hide-time, show-time, time-until-hide, middle-click-action, zoom-effect
		let Monitor = this._getMonitor(); //keep properties up to date
		
		this._CONF_onlyScreenWindows = this.settings.get_boolean("only-screen-windows");
		this._CONF_onAllScreens = this.settings.get_boolean("on-all-screens");
		
		this._CONF_autohide = this.settings.get_boolean("autohide");
		this._CONF_hideTime = this.settings.get_double("hide-time");
		this._CONF_showTime = this.settings.get_double("show-time");
		this._CONF_menuTimeout = this.settings.get_int("time-until-hide");
		this._CONF_dontHideSize = this.settings.get_int("dont-hide-size");
		if(this._CONF_dontHideSize < 0)
			this._CONF_dontHideSize = 0;
		else if(this._CONF_dontHideSize > this._CONF_iconSize-1)
			this._CONF_dontHideSize = this._CONF_iconSize-1;
		
		this._CONF_zoomEffect = this.settings.get_boolean("zoom-effect");
		this._CONF_cutStruts = this.settings.get_boolean("cut-view");
		this._CONF_dockY_type = this.settings.get_enum("dock-y");
		this._CONF_middleClick = this.settings.get_enum("middle-click-action");
		this._CONF_showWindowTitle = this.settings.get_boolean("show-window-title");
		this._CONF_showAllWS = this.settings.get_boolean("list-all-workspaces");
		this._CONF_showWSline = this.settings.get_boolean("show-workspace-number");
		this._CONF_expandBox = this.settings.get_boolean("expand-box");
		this._CONF_fixDockPositionAt = this.settings.get_enum("fix-docklet-position-at");
		
		this._CONF_iconSize = Math.max(this.settings.get_int("basic-icon-size"), 10);
			
		this._CONF_buttonMaxWidth = this.settings.get_int("basic-button-width");
		if(this._CONF_buttonMaxWidth < 10)
			this._CONF_buttonMaxWidth = 10;
		else if(this._CONF_buttonMaxWidth > this._DATA_inner_width)
			this._CONF_buttonMaxWidth = this._DATA_inner_width;
		
		this._CONF_dockX_percent = this.settings.get_double("dock-x-percent");
		this._CONF_dockWidth_percent = this.settings.get_double("dock-width-percent");
		
		//----------DATA---------->
		
		this._DATA_monitorWidth = Monitor.width;
		
		this._DATA_space_left = DOCK_PADDING_LEFT + (this._CONF_showWSline ? WS_LABEL_WIDTH : 0);
		this._DATA_padding_x_sum = this._DATA_space_left + DOCK_PADDING_RIGHT;
		this._DATA_wsList_space = Math.round(this._CONF_iconSize/3);
		this._DATA_real_x = Monitor.x + Math.round(Monitor.width * this._CONF_dockX_percent);
		
		//_dockHeight, _dockRealWidth: to keep properties in a variable
		this._dockHeight = this._CONF_iconSize;
		if(this._CONF_expandBox) {
			this._DATA_inner_width = this._DATA_monitorWidth - this._DATA_padding_x_sum;
			this._DATA_min_width = this._dockRealWidth = Math.round(this._DATA_monitorWidth * this._CONF_dockWidth_percent) - this._DATA_padding_x_sum;
		}
		else {
			this._DATA_inner_width = Math.round(this._DATA_monitorWidth * this._CONF_dockWidth_percent) - this._DATA_padding_x_sum;
			this._dockRealWidth = this._DATA_inner_width + this._DATA_padding_x_sum;
		}
		
		
		let style_class;
		if(this._CONF_dockY_type == TOP) {
			this._DATA_icons_ListedTo_default = 0;
			
			if(this._CONF_cutStruts)
				this._dockY = IF_STRUT_AND_TOP_Y;
			else
				this._dockY = 0;
			this._DATA_iconSpacingTop = 0;
			this._hideTo = this._CONF_dontHideSize - this._CONF_iconSize;
			style_class = "panelDocklet top";
		}
		else {
			this._DATA_icons_ListedTo_default = DOCK_PADDING_BOTTOM;
			
			this._DATA_monitorHeight = Monitor.height;
			this._DATA_iconSpacingTop = DOCK_PADDING_BOTTOM;
			this._dockY = this._DATA_monitorHeight - this._dockHeight - DOCK_PADDING_BOTTOM;
			this._hideTo = this._dockY + (this._dockHeight - this._CONF_dontHideSize);
			style_class = "panelDocklet bottom";
		}
		
		
		if(this._CONF_showWindowTitle) {
			this._DATA_max_icons = Math.floor(this._DATA_inner_width / this._CONF_iconSize);
			this._DATA_max_buttons = Math.floor(this._DATA_inner_width / this._CONF_buttonMaxWidth);
		}
		else
			this._DATA_max_icons = this._DATA_max_buttons = Math.floor(this._DATA_inner_width / this._CONF_iconSize);
		
		this._DATA_iconTextureSize = (this._CONF_zoomEffect ? this._CONF_iconSize*2 : this._CONF_iconSize) - ICON_PADDING_SUM;
		this._DATA_iconLabel_fontSize = Math.round(this._CONF_iconSize/3);
		this._DATA_iconLabel_x = this._CONF_iconSize - ICON_PADDING_SUM + ICON_LABEL_SPACE;
		this._DATA_iconLabel_y = Math.round(this._CONF_iconSize/3) - ICON_PADDING_TOP-1;
		this._DATA_iconLabel_minWidth = this._CONF_iconSize *2;
		
		
		return style_class;
	},
	_onMouseClick: function(actor, event) {
		if(event.get_button()==1) {
			let wsI = this._box.getWSunderMousePos(global.get_pointer()[1]);
			
			if(wsI != this._box._currentWS)
				global.screen.get_workspace_by_index(wsI).activate(global.get_current_time());
			
			return;
		}
		else if(this._menuIsOpened || this._isHidden || event.get_button() != 3)
			return;
		this._menuIsOpened = true;
		
		if(!this._menu)
			this._menu = new DockletMenu(this);
		this._menu.popup();
	},
	
	_addStrut: function() {
		//FIXME: there must be a better way... Lets look up the needed commands in the documentation. It was here somewhere, I know it...
		let height = this._dockHeight + DOCK_PADDING_BOTTOM,
			y = this._dockY;
		
		if(this._CONF_dockY_type == TOP) {
			y = 0;
			height += IF_STRUT_AND_TOP_Y;
		}
		this._cutStrut = new St.Bin({
				x: this._getMonitor().x,
				y: y,
				width: this._DATA_monitorWidth,
				height: height
			});
		
		LayoutManager.addChrome(this._cutStrut, { affectsStruts: true });
		this._cutStrut.hide();
	},
	_saveHotCorners: function() {
		if(this._CONF_dockY_type == TOP && !this._CONF_cutStruts &&
				(this._calcDockX(this._dockRealWidth) <= 0 || (this._CONF_expandBox && this._CONF_fixDockPositionAt == RIGHT))) {
			++this._DATA_real_x;
			--this._DATA_monitorWidth;
			this.actor.set_x(this._calcDockX(this._dockRealWidth));
		}
		
		if(this._CONF_dockY_type == BOTTOM && !this._CONF_autohide)
			LayoutManager.trayBox.raise(this.actor);
		else if(this._CONF_dockY_type == BOTTOM)
			LayoutManager.trayBox.lower(this.actor);
	},
	
	_removeMenuTimeout: function() {
		if(this._ID_menuTimeout) {
			Mainloop.source_remove(this._ID_menuTimeout);
			this._ID_menuTimeout = false;
		}
	},
	_hideDock: function() {
		if(this._menuIsOpened || this._ID_menuTimeout || this._isHidden)
			return;
		
		this._isHidden=true;
		this._ID_menuTimeout = Mainloop.timeout_add(this._CONF_menuTimeout, Lang.bind(this, function() {
				if(this._CONF_menuTimeout && this._box._WSareListed)
					this._box.unlistWS();
				else {
					let s = this._dockHeight;
					Tweener.addTween(this.actor, {
							y: this._hideTo,
							time: this._CONF_hideTime,
							transition: "easeOutQuad"
						});
				}
				
				this.stopZooming();
				
				this._ID_menuTimeout = false;
			}));
	},
	_showDock: function(actor, event) {
		if(this._CONF_zoomEffect)
			this.startZooming();
		
		if(!this._isHidden)
			return;
		this._removeMenuTimeout();
		this._isHidden=false;
		let ws = this._box._workspaces;
		if(this._CONF_showAllWS && (ws.length>2 || (ws.length==2 && ws[1]._windows.length) || this._box._currentWS))
			this._box.listWS();
		else 
			Tweener.addTween(this.actor, {
					y: this._dockY,
					time: this._CONF_showTime,
					transition: "easeOutQuad"
				});
	},
	
	startZooming: function() {
		if(this._ID_mouseTracking)
			return;
		
		this._ID_mouseTracking = Mainloop.timeout_add(
				MOUSE_POLL_FREQUENCY,
				Lang.bind(this, function() {
						let pointer = global.get_pointer(),
							x = pointer[0] - this.actor.x,
							y = pointer[1] - this.actor.y,
							
							wsI = this._box.getWSunderMousePos(y);
						
						if(this._lastZoomWS != wsI) {
							if(this._lastZoomWS != null)
								this._box._workspaces[this._lastZoomWS].undoIconsZoom();
							this._box._workspaces[this._lastZoomWS = wsI]._windows.forEach(function(o) {o.actor.raise_top();}); //Variable is changed here!
						}
						
						this._box._workspaces[wsI].zoomIcons(x);
						return true;
					})
			);
		
	},
	stopZooming: function(actor, event) {
		if(this._menuIsOpened) {
			if(this._ID_mouseTracking)
				Mainloop.source_remove(this._ID_mouseTracking);
			return;
		}
		if(event) {//stopZooming is not allways called by listener!
			let related = event.get_related(),
				actor = this.actor;
			if(related == actor || related.get_parent() == actor)
				return;
		}
		if(this._ID_mouseTracking) {
			Mainloop.source_remove(this._ID_mouseTracking);
			if(this._lastZoomWS != null)
				this._box._workspaces[this._lastZoomWS].undoIconsZoom();
			
			this._ID_mouseTracking = false;
			this._lastZoomWS = null;
			this._ID_menuTimeout = false;
		}
	},
	
	setDockHeight: function(s, yPos, show) {
		if(s && s != this._dockHeight) {
			this._dockHeight = s;
			if(this._CONF_showWSline && s<=this._CONF_iconSize)
				this._box._setWSlabelPadding(s);
		}
		else
			s = this._dockHeight;
		
		Tweener.addTween(this.actor, {
				height: DOCK_PADDING_BOTTOM + s,
				y: yPos || this._dockY,
				time: ((yPos && !show) ? this._CONF_hideTime : this._CONF_showTime),
				transition: "easeOutQuad"
			});
		
	},
	setDockWidth: function(w) {
		this._dockRealWidth = w;
		Tweener.addTween(this.actor, {
				x: this._calcDockX(w),
				width: w,
				time: 0.3,
				transition: "easeInQuart"
			});
	},
	setExpandedDockWidth: function(l, winBox) {
		if(this._box._WSareListed && l < this._winLenRecord) {
			l = this._winLenRecord;
			winBox = this._winLenRecordClass;
		}
		
		if(l < this._DATA_max_icons && (!this._CONF_showWindowTitle || l < this._DATA_max_buttons)) {
			let w = l * (this._CONF_showWindowTitle ? this._CONF_buttonMaxWidth : this._CONF_iconSize) + this._DATA_padding_x_sum;
			if(w < this._DATA_min_width)
				w = this._DATA_min_width;
			this.setDockWidth(w);
		}
		else {
			this.setDockWidth(this._DATA_monitorWidth);
			if(l >= this._DATA_max_icons)
				winBox._setIconSize(l);
			
			else
				winBox._setButtonSize(l);
		}
	},
	_calcDockX: function(w) {
		switch(this._CONF_fixDockPositionAt) {
			case RIGHT:
				return this._DATA_real_x - w;
			case MIDDLE:
				return this._DATA_real_x - Math.ceil(w/2);
			default:
				return this._DATA_real_x;
		}
	},
	
	_update: function() {
		//autohide, dont-hide-size
		if(this._box._WSareListed)
			this._box.unlistWS();
		
		
		let style_class = this._loadConfigs();
		
		
		//autohide, dont-hide-size
		if(this._CONF_autohide) {
			this._isHidden = false; //if dock was hidden before reload
			this._hideDock();
		}
		else if(this._isHidden) {
			this._showDock();
			if(this._CONF_zoomEffect)
				this.stopZooming(); //because script thinks, function was called because of mouseover
		}
		else if(this._CONF_showAllWS)
			this._box.listWS();
		
		
		//dock-y
		this.actor.set_style_class_name(style_class);
		this.setDockHeight();
		this._saveHotCorners();
		
		
		let showLabels = this._CONF_showWindowTitle,
			isBottom = (this._CONF_dockY_type==BOTTOM),
			w = this._box._workspaces,
			i = w.length,
			win, winLen;
		
		//show-window-title
		while(i--) {
			win = w[i];
			win._iconSize = this._CONF_iconSize;
			winLen = win._windows.length;
			
			win.reloadAllLabels(showLabels);
			
			//this is for the case _updateIconSize isnt needed but icon-/buton-size has changed
			if(this._CONF_showWindowTitle)
				win._setButtonSize(winLen);
			else
				win._setIconSize(winLen);
			
			//this is for the case _setIconSize is needed but _CONF_showWindowTitle is enabled
			win._updateIconSize(winLen);
			
			//dock-y
			if(isBottom)
				win.moveIcons(DOCK_PADDING_BOTTOM);
			else
				win.moveIcons(0);
		}
		
		//dock-width-percent. dock-x-percent, fix-docklet-position-at
		if(this._CONF_expandBox) {
			let ws = w[this._box._currentWS];
			this.setExpandedDockWidth(ws._windows.length, ws);
		}
		else
			this.setDockWidth(this._dockRealWidth);
		
		
		//autohide, zoom-effect
		if(this._ID_leaveEvent) {// && this._ID_enterEvent
			this.actor.disconnect(this._ID_leaveEvent);
			this.actor.disconnect(this._ID_enterEvent);
			this._ID_leaveEvent = false;
		}
		if(this._CONF_autohide) {
			this._ID_enterEvent = this.actor.connect("enter-event", Lang.bind(this,this._showDock));
			this._ID_leaveEvent = this.actor.connect("leave-event", Lang.bind(this, this._hideDock));
		}
		else if(this._CONF_zoomEffect) {
			this._ID_enterEvent = this.actor.connect("enter-event", Lang.bind(this, this.startZooming));
			this._ID_leaveEvent = this.actor.connect("leave-event", Lang.bind(this, this.stopZooming));
		}
		
		if(this._menu) {
			this._menu._destroy();
			this._menu = false;
		}
		
		//cut-strut
		if(this._cutStrut) {
			this._cutStrut.destroy();
			this._cutStrut = false;
			if(this._CONF_cutStruts)
				this._addStrut();
		}
		else if(this._CONF_cutStruts)
			this._addStrut();
	},
	_reload: function() {
		//list-all-workspaces, show-workspace-number, expand-box, basic-icon-size
		//ignoring _currentWSlabel-padding, when _CONF_showAllWS gets enabled and icons are shrinked because of y-pos
		if(this._box)
			this._box._destroy();
		
		let style_class = this._loadConfigs();
		
		//dock-y
		this._saveHotCorners();
		
		this.actor.set_style_class_name(style_class);
		
		if(this._CONF_expandBox) {
			let ws = this._box._workspaces[global.screen.get_active_workspace_index()];
			this.setExpandedDockWidth(ws._windows.length, ws);
		}
		else
			this.setDockWidth(this._dockRealWidth);
		this.setDockHeight(this._dockHeight);
		
		this._box = new workspaceBox(this);
		this._box._indexAllWS();
		
		if(this._menu) {
			this._menu._destroy();
			this._menu = false;
		}
		
		
		//just to be sure:
		
		//autohide, dont-hide-size
		if(this._CONF_autohide) {
			this._isHidden = false; //if dock was hidden before reload
			this._hideDock();
		}
		else if(this._isHidden) {
			this._showDock();
			if(this._CONF_zoomEffect)
				this.stopZooming(); //script asumes, function was called after mouseover
		}
		else if(this._CONF_showAllWS)
			this._box.listWS();
		
		
		//autohide, zoom-effect
		if(this._ID_leaveEvent) {// && this._ID_enterEvent
			this.actor.disconnect(this._ID_leaveEvent);
			this.actor.disconnect(this._ID_enterEvent);
			this._ID_leaveEvent = false;
		}
		if(this._CONF_autohide) {
			this._ID_enterEvent = this.actor.connect("enter-event", Lang.bind(this,this._showDock));
			this._ID_leaveEvent = this.actor.connect("leave-event", Lang.bind(this, this._hideDock));
		}
		else if(this._CONF_zoomEffect) {
			this._ID_enterEvent = this.actor.connect("enter-event", Lang.bind(this, this.startZooming));
			this._ID_leaveEvent = this.actor.connect("leave-event", Lang.bind(this, this.stopZooming));
		}
		
		
		//cut-strut
		if(this._cutStrut) {
			this._cutStrut.destroy();
			this._cutStrut = false;
			if(this._CONF_cutStruts)
				this._addStrut();
		}
		else if(this._CONF_cutStruts)
			this._addStrut();
	},
	_destroy: function() {
		/* as long as I am using my own class. No disconnect needed for configs
		let l = this._ID_settingsListener,
			i = l.length;
		while(i--)
			this.settings.disconnect(l[i]);*/
		
		if(this._ID_leaveEvent) {// && this._ID_enterEvent
			this.actor.disconnect(this._ID_leaveEvent);
			this.actor.disconnect(this._ID_enterEvent);
		}
		this.actor.disconnect(this._ID_clickEvent);
		this._removeMenuTimeout();
		
		if(this._ID_mouseTracking)
			Mainloop.source_remove(this._ID_mouseTracking);
		
		this._dragPointer.destroy();
		this._box._destroy();
		this.actor.destroy();
		
		if(this._cutStrut) {
			this._cutStrut.destroy();
			this._cutStrut = false;
		}
	}
}


function workspaceBox(dock) {
	this._init(dock);
}
workspaceBox.prototype = {
	_init: function(dock) {
		this._dock = dock;

		this._WSareListed = false;
		this._currentWindowFocus = false;
		this._workspaces = [];
		this._allWsWindows = [];
		this._zoomedIcons = [];
		this._currentWS = global.screen.get_active_workspace_index();
		
		if(dock._CONF_showWSline) {
			this._wsLabels = [];
			
			this._currentWSlabel = new St.Label({text:"1", x:1, y:1, width:WS_LABEL_WIDTH, style_class:"wsLabelMain"});
			dock.actor.add(this._currentWSlabel);
			this._setWSlabelPadding();
		}
			
		if(dock._CONF_showAllWS)
			this._wsbg = new St.Bin({style_class: "wsListBack", x:dock._DATA_space_left, width:dock._DATA_inner_width});
		
		
		//Consider:
		//_CONF_onlyScreenWindows can be enabled without _CONF_onAllScreens enabled!
		//and _CONF_onlyScreenWindows can be disabled for primary-Docklet but not for others
		if(dock._CONF_onlyScreenWindows || dock._CONF_onAllScreens) {
			if(dock._monitorId == "primary") {
				this._ID_windowEnteredMonitor = global.screen.connect('window-entered-monitor', Lang.bind(this, this._onWindowEnteredMonitor));
				this._ID_windowLeftMonitor = global.screen.connect('window-left-monitor', Lang.bind(this, this._onWindowLeftMonitor));
			}
			
			this._realMonitorId = ((dock._monitorId == "primary") ? LayoutManager.primaryIndex : dock._monitorId);
		}
		
		this._ID_newWorkspace = global.screen.connect("notify::n-workspaces", Lang.bind(this, this._newWS));
		this._ID_switchWorkspace = global.window_manager.connect("switch-workspace", Lang.bind(this, this._changeWS));
		
		this._ID_minimize = global.window_manager.connect("minimize", Lang.bind(this, this._onMinimize));
		this._ID_map = global.window_manager.connect("map", Lang.bind(this, this._onMap));
		
		let tracker = Shell.WindowTracker.get_default();
		this._ID_focusApp = tracker.connect("notify::focus-app", Lang.bind(this, this._onFocus));
	},
	getWSunderMousePos: function(y) {
		let dock = this._dock,
			iconSize = dock._CONF_iconSize,
			startY = ((dock._CONF_dockY_type == TOP) ? iconSize + dock._DATA_wsList_space : 0),
			
			ws = this._workspaces,
			wsLast = ws.length-1,
			wsI;
		
		if(!this._WSareListed || y < startY || (!startY && y > wsLast*iconSize))
			wsI = this._currentWS;
		else {
			wsI = Math.floor((y - startY) / (iconSize + WS_LINES_SPACE_BETWEEN));
			if(wsI >= this._currentWS)
				++wsI;
			
			if(wsI > wsLast)
				wsI = wsLast;
			//else(wsI < 0) -> not necessary because of: if(... || y < startY || ...)
		}
		
		return wsI;
	},
	
	_setWSlabelPadding: function(s) {
		this._currentWSlabel.set_style("padding: "+Math.round(((s || this._dock._CONF_iconSize) - DOCK_WS_LABEL_FONT_SIZE) / 2 - 2)+"px 0;");
	},
	setRecordWS: function() {
		let dock = this._dock,
			ws = this._workspaces,
			i = ws.length,
			record = 0,
			recordClass = null,
			current_ws;
		while(i--) {
			current_ws = ws[i];
			if(current_ws._windows.length > record) {
				record = current_ws._windows.length;
				recordClass = current_ws;
			}
		}
		dock._winLenRecord = record;
		dock._winLenRecordClass = recordClass;
		if(dock._CONF_expandBox && this._WSareListed)
			dock.setExpandedDockWidth(record, recordClass);
	},
	
	_indexAllWS: function() {
		let i = global.screen.n_workspaces;
		
		while(i--) {
			this._workspaces[i] = new windowBox(global.screen.get_workspace_by_index(i), this, this._dock);
		}
		if(this._dock._CONF_showWSline)
			this._currentWSlabel.text = (global.screen.get_active_workspace_index()+1).toString();
	},
	_newWS: function(ws, b) {
		//code-parts from: usr/share/gnome-shell/js/ui/main.js
		let oldNumWorkspaces = this._workspaces.length,
			newNumWorkspaces = global.screen.n_workspaces;
		
		if(newNumWorkspaces > oldNumWorkspaces) {
			// Assume workspaces are only added at the end
			for(let i = oldNumWorkspaces; i < newNumWorkspaces; ++i)
				this._workspaces[i] = new windowBox(global.screen.get_workspace_by_index(i), this, this._dock);
		}
		else if(newNumWorkspaces < oldNumWorkspaces){
			// Assume workspaces are only removed sequentially
			// (e.g. 2,3,4 - not 2,4,7)
			let removedNum = oldNumWorkspaces - newNumWorkspaces,
				removedIndex;
			for(let i=0; i < oldNumWorkspaces; ++i) {
				if(this._workspaces[i]._myWorkspace != global.screen.get_workspace_by_index(i)) {
					removedIndex = i;
					break;
				}
			}
			//especially needed for session-start... Strange behavior...
			if(this._currentWS >= newNumWorkspaces) {
				let to = newNumWorkspaces-1;
				this._changeWS(global.screen.get_workspace_by_index(to), this._currentWS, to);
			}
			
			let lostWorkspaces = this._workspaces.splice(removedIndex, removedNum);
			lostWorkspaces.forEach(function(winBox) {
					winBox._destroy();
				});
		}
		
		//no idea why this is necessary.
		//But multiple app-windows on different workspaces wont be recognized properly without reconnect...
		let i = this._workspaces.length;
		while(i--) {
			this._workspaces[i]._removeListener();
			this._workspaces[i]._setListener();
		}
		
		if(this._WSareListed) {
			this.unlistWS();
			this.listWS();
		}
		
//		return false; //blind copy
	},
	_changeWS: function(wm, from, to, direction) {
		this._currentWS = to;
		
		let movesUp = (to>from) ? true : false,
			toWS = this._workspaces[to];
		
		if(to >= this._workspaces.length)
			toWS = new windowBox(global.screen.get_workspace_by_index(i), this, this._dock);
		
		if(this._WSareListed) {
			let dock = this._dock,
				confWSline = dock._CONF_showWSline,
				isBottom = (dock._CONF_dockY_type == BOTTOM),
				lineHeight = dock._CONF_iconSize + WS_LINES_SPACE_BETWEEN,
				startPos = (isBottom ? DOCK_PADDING_BOTTOM - dock._CONF_iconSize : dock._DATA_wsList_space) - WS_LINES_SPACE_BETWEEN;
			
			for(let nFrom=from, loop = (movesUp ? 1 : -1); nFrom != to; nFrom+=loop) {
				this._workspaces[nFrom].moveIcons((nFrom + (movesUp ? 1 : 0)) * lineHeight + startPos);
				if(confWSline)
					this._wsLabels[(nFrom - (movesUp ? 0 : 1))].text = ((nFrom+1)+":").toString();
			}
			
			this._workspaces[to].moveIcons(
					(isBottom ? this._calcListedBottomPos(to) : false)
				);
			
			this._wsbg.set_height(to*lineHeight);
		}
		else {
			this._workspaces[from].removeAllIcons(movesUp);
			toWS.activateIcons(!movesUp);
			
			toWS._updateHeight();
		
			if(this._dock._CONF_expandBox)
				this._dock.setExpandedDockWidth(toWS._windows.length, toWS);
		}
		if(this._dock._CONF_showWSline)
			this._currentWSlabel.text = (to+1).toString();
	},
	_calcListedBottomPos: function(i) {
		let ws = this._workspaces,
			confIconSize = this._dock._CONF_iconSize,
			lineHeight = confIconSize + WS_LINES_SPACE_BETWEEN;
		
		return (ws.length-1)*lineHeight + DOCK_PADDING_BOTTOM + this._dock._DATA_wsList_space + (confIconSize - ws[i]._iconSize);
	},
	_calcListY: function() {
		let confIconSize = this._dock._CONF_iconSize;
		return confIconSize + Math.round(confIconSize/3);
	},
	listWS: function() {
		if(this._WSareListed)
			return;
		this._WSareListed = true;
		
		let dock = this._dock,
			ws = this._workspaces,
			wsLen = ws.length,
			cI = this._currentWS,
			
			confShowWSline = dock._CONF_showWSline,
			isBottom = (dock._CONF_dockY_type == BOTTOM),
			actor = this._dock.actor,
		
			wsLabels = this._wsLabels,
			start_sum = (isBottom ? DOCK_PADDING_BOTTOM : dock._CONF_iconSize + dock._DATA_wsList_space),
			bg_height = 0,
			sum = start_sum,
			sumAdd = dock._CONF_iconSize + WS_LINES_SPACE_BETWEEN,
			labelOffset = Math.round(dock._CONF_iconSize/4),
			dockHeight = wsLen*sumAdd + dock._DATA_wsList_space,
			item;
		
		
		if(dock._CONF_expandBox)
			dock.setExpandedDockWidth(0);
		dock.setDockHeight(dockHeight, (isBottom ? dock._DATA_monitorHeight - dockHeight : false), true);
		
		
		for(let i=0, max = ws.length; i<max; ++i) {
			if(i==cI) {
				if(isBottom) {
					let pos = this._calcListedBottomPos(i);
					ws[i].moveIcons(pos);
					
					if(confShowWSline)
						this._currentWSlabel.set_y(pos);
				}
				
				bg_height = sum;
				continue;
			}
			
			ws[i].activateIcons(true, sum);
			
			if(confShowWSline) {
				item = new St.Label({text:(i+1)+":", x:WS_LINES_SPACE_LEFT, y:sum + labelOffset});
				actor.add(item);
				wsLabels.push(item);
			}
			sum += sumAdd;
		}
		
		
		item = this._wsbg;
		actor.add(item);
		item.set_y(start_sum);
		item.set_size(dock._dockRealWidth - dock._DATA_padding_x_sum, bg_height-start_sum);
		item.lower_bottom();
	},
	unlistWS: function(cI) {
		if(!this._WSareListed)
			return;
		
		this._WSareListed = false;
		
		cI = ((cI!=undefined) ? cI : this._currentWS); //0==false
		let dock = this._dock,
			ws = this._workspaces,
			wsI = ws.length;
		
		//if new WS has been created or removed, ws.length and labels.length can be different
		if(dock._CONF_showWSline) {
			let labels = this._wsLabels,
				laI = labels.length;
			
			while(laI--) {
				labels[laI].destroy();
			}
			this._wsLabels = [];
			
			
			if(dock._CONF_dockY_type == BOTTOM)
				this._currentWSlabel.set_y(1);
		}
		dock.actor.remove_actor(this._wsbg);
		
		ws[cI]._updateHeight();
		if(dock._CONF_dockY_type == BOTTOM)
			ws[cI].moveIcons(dock._DATA_iconSpacingTop);
		
		while(wsI--) {
			if(wsI != cI)
				ws[wsI].removeAllIcons(true, true);
		}
		
		if(dock._CONF_expandBox)
			dock.setExpandedDockWidth(ws[cI]._windows.length, ws[cI]);
	},
	
	_onWindowEnteredMonitor: function(metaScreen, monitorIndex, metaWin) {
		//Asuming: new windows will be added BEFORE monitor-entering
		//so we can find out the monitor without getting into trouble (double icons),
		//which seem not to be possible outside this function
		//maybe I should travel to a far distand land, to find that holy documentation the elders are talking about...
		if(!metaWin.PANEL_DOCKLET_monitor) {
			metaWin.PANEL_DOCKLET_monitor = monitorIndex;
		
			if(monitorIndex != this._realMonitorId) {
				if(this._dock._CONF_onAllScreens) {
					let ws = metaWin.get_workspace(),
						i = ws.index(),
						otherDock = this._dock._secondaryPanelDocklets[monitorIndex];
					
					//otherDock can be undefined, when primary ws has changed
					//(docklet will be reloaded afterwards and the old listener will be called)
					if(otherDock && otherDock._CONF_onlyScreenWindows)
						otherDock._box._workspaces[i]._onWindowAdded(ws, metaWin);
				}
				
				return;
			}
			
		}
		metaWin.PANEL_DOCKLET_monitor = monitorIndex;
		
		if(!this._dock._CONF_onlyScreenWindows)
			return;//window was just moved or
		
		let ws = metaWin.get_workspace(),
			i = ws.index();
		this._workspaces[i]._onWindowAdded(ws, metaWin);
	},
	_onWindowLeftMonitor: function(metaScreen, monitorIndex, metaWin) {
		if(monitorIndex != this._realMonitorId) {
			if(this._dock._CONF_onAllScreens) {
				let ws = metaWin.get_workspace(),
					i = ws.index(),
					otherDock = this._dock._secondaryPanelDocklets[monitorIndex];
				if(otherDock._CONF_onlyScreenWindows)
					otherDock._box._workspaces[i]._onWindowRemoved(ws, metaWin);
			}
			return;
		}
		if(!this._dock._CONF_onlyScreenWindows)
			return;//window was just moved or
		
		let ws = metaWin.get_workspace();
		
		if(!ws)//when window ist closed
			return;
			
		let i = ws.index();
		
		this._workspaces[i]._onWindowRemoved(ws, metaWin);
	},
	_onFocus: function() {
		if(this._currentWindowFocus) {
			this._currentWindowFocus._myWindowClass._checkFor_allWSWindow(this._currentWindowFocus);
			this._currentWindowFocus.removeFocus();
		}
		
		let f = global.display.focus_window, f_icon;
		if(f && (f_icon = f[this._dock._winVarKey_icon])) {
			//we dont have to worry about all-workspace-windows. Just focus the first icon
			//and it will focus its doubles (including current workspace) too
			f_icon.showFocused();
			f_icon._myWindowClass._checkFor_allWSWindow(f_icon);
			
			this._currentWindowFocus = f_icon;
		}
		else
			this._currentWindowFocus = false;
	},
	_onMinimize: function(shellwm, actor) {
		actor.get_meta_window()[this._dock._winVarKey_icon].showMinimized();
	},
	_onMap: function(shellwm, actor) {
		let a;
		if((a=actor.get_meta_window()[this._dock._winVarKey_icon]))
			a.showMaped();
	},
	
	_addAllWsWindow: function(app, win, notThisWS) {
		let winVarKey_addedToAll = this._dock._winVarKey_addedToAll;
		
		if(!win[winVarKey_addedToAll]) {
			this._allWsWindows.push([app, win]);
			
			let ws = this._workspaces,
				wsI= ws.length;
			if(notThisWS) {
				while(wsI--) {
					if(ws[wsI]._myWorkspace != notThisWS)
						ws[wsI].addWindow(app, win);
				}
			}
			else {
				while(wsI--)
					ws[wsI].addWindow(app, win);
			}
			win[winVarKey_addedToAll] = true;
			return true;
		}
		return false
	},
	_removeAllWsWindow: function(win, notAllWS) {
		let a = this._allWsWindows,
			i = a.length,
			
			ws = this._workspaces,
			wsI= ws.length;
			
		delete win[this._dock._winVarKey_addedToAll];
		if(notAllWS) {
			let notIndex = win.get_workspace().index(),
				winVarKey_icon = this._dock._winVarKey_icon,
				icon = win[winVarKey_icon],
				lastIcon = (icon._myDoubles[notIndex] ? icon._myDoubles[notIndex] : icon);
			
			lastIcon._myDoubles = [];
			win[winVarKey_icon] = lastIcon;
			
			
			while(wsI--) {
				if(wsI != notIndex)
					ws[wsI].removeWindow(win);
			}
		}
		else {
			while(wsI--)
				ws[wsI].removeWindow(win);
		}
		
		
		while(i--) {
			if(a[i][1] != win)
				continue;
			
			this._allWsWindows.splice(i, 1);
			return true;
		}
		return false;
	},
	
	_destroy: function() {
		global.screen.disconnect(this._ID_newWorkspace);
		global.window_manager.disconnect(this._ID_switchWorkspace);
		global.window_manager.disconnect(this._ID_minimize);
		global.window_manager.disconnect(this._ID_map);
		Shell.WindowTracker.get_default().disconnect(this._ID_focusApp);
		
		if(this._ID_windowEnteredMonitor) {
			global.screen.disconnect(this._ID_windowEnteredMonitor);
			global.screen.disconnect(this._ID_windowLeftMonitor);
			this._ID_windowEnteredMonitor = false;
		}
		
		if(this._wsbg)
			this._wsbg.destroy();
		
		if(this._currentWSlabel)
			this._currentWSlabel.destroy();
		if(this._wsLabels) {
			let labels = this._wsLabels,
				i = labels.length;
			while(i--) {
				labels[i].destroy();
			}
		}
			
		
		let i = this._workspaces.length;
		while(i--) {
			this._workspaces[i]._destroy();
		}
	}
}


function windowBox(ws, wsBox, dock) {
	this._init(ws, wsBox, dock);
}
windowBox.prototype = {
	_init: function(ws, wsBox, dock) {
		this._dock = dock;
		this._myWorkspace = ws;
		this._wsBox = wsBox;
		this._windows = [];
		this._isListedTo = dock._DATA_icons_ListedTo_default;
		
		this._iconSize = dock._CONF_iconSize;
		this._buttonWidth = dock._CONF_showWindowTitle ? dock._CONF_buttonMaxWidth : dock._CONF_iconSize;
		this._showLabel = dock._CONF_showWindowTitle;
		
		this._ID_windowAdded = false;
		this._ID_windowRemoved = false;
		
		
		this._indexAllWindows();
		
		this._setListener();
	},
	_indexAllWindows: function() {
		let realWindows = this._myWorkspace.list_windows(),
			i = realWindows.length;
		
		while(i--) {
			this._onWindowAdded(this._myWorkspace, realWindows[i]);
			
			if(realWindows[i].has_focus()) 
				this._wsBox._currentWindowFocus = this._windows[this._windows.length-1];//last element is currently added window
		}
	},
	_setListener: function() {
		let ws = this._myWorkspace;
		this._ID_windowRemoved = ws.connect("window-removed", Lang.bind(this, this._onWindowRemoved));
		this._ID_windowAdded = ws.connect("window-added", Lang.bind(this, this._onWindowAdded));
	},
	_removeListener: function() {
		let ws = this._myWorkspace;
		
		if(this._ID_windowAdded) { //&& this._ID_windowRemoved
			ws.disconnect(this._ID_windowAdded);
			ws.disconnect(this._ID_windowRemoved);
			this._ID_windowAdded = false; //this._ID_windowRemoved = false;
		}
	},
	
	_onWindowAdded: function(metaWorkspace, metaWindow) {
		if(this._dock._CONF_onlyScreenWindows && metaWindow.PANEL_DOCKLET_monitor != this._wsBox._realMonitorId)
			return;
		
		let tracker = Shell.WindowTracker.get_default(),
			app = tracker.get_window_app(metaWindow);
		
		if(app && tracker.is_window_interesting(metaWindow)) {
			if(!metaWindow.is_on_all_workspaces() || !this._wsBox._addAllWsWindow(app, metaWindow))
				this.addWindow(app, metaWindow);
			
			if(this._wsBox._WSareListed && this._dock._CONF_expandBox)
				this._wsBox._wsbg.set_width(this._dock._dockRealWidth - this._dock._DATA_padding_x_sum);
		}
	},
	addWindow: function(app, win) {
		this._addIcon(app, win);
		this._updateIconSize();
	},
	_onWindowRemoved: function(metaWorkspace, metaWindow) {
		if(this._dock._CONF_onlyScreenWindows && metaWindow.PANEL_DOCKLET_monitor != this._wsBox._realMonitorId)
			return;//wouldnt hurt to try, but wont find it anyway
		
		if(!metaWindow[this._dock._winVarKey_addedToAll] || !this._wsBox._removeAllWsWindow(metaWindow))
			this.removeWindow(metaWindow);
		
		if(this._wsBox._WSareListed && this._dock._CONF_expandBox)
			this._wsBox._wsbg.set_width(this._dock._dockRealWidth - this._dock._DATA_padding_x_sum);
	},
	removeWindow: function(win) {
		let i = this._windows.length;
		while(i--) {
			if (this._windows[i]._myWindow == win) {
				
				this.deleteIcon(i);
				
				if(this._dock._CONF_expandBox && this == this._dock._winLenRecordClass)
					this._wsBox.setRecordWS();
				
				
				return;
			}
		}
	},
	
	_checkFor_allWSWindow: function(win) {
		let realwin = win._myWindow;
		
		if(realwin.is_on_all_workspaces()) {
			if(!realwin[this._dock._winVarKey_addedToAll])
				this._wsBox._addAllWsWindow(win._myApp, realwin, this._myWorkspace);
		}
		else if(realwin[this._dock._winVarKey_addedToAll])
			this._wsBox._removeAllWsWindow(realwin, true);
	},
	
	_addIcon: function(app, current_window) {
		let dock = this._dock,
		
			item = new windowIconButton(
				dock,
				app,
				current_window,
				this,
				this._windows.length, //index-count is current index
				((this._wsBox._WSareListed || this._myWorkspace.index() == dock._box._currentWS) ? this._iconSize :false),
				this._buttonWidth,
				this._isListedTo
			),
			winVarKey_icon = dock._winVarKey_icon;
		
		this._windows.push(item);
		if(dock._CONF_expandBox && this._windows.length > dock._winLenRecord) {
			dock._winLenRecord = this._windows.length;
			dock._winLenRecordClass = this;
		}
		
		if(this._showLabel)
			item.addLabel();
				
		if(current_window[winVarKey_icon] && current_window[winVarKey_icon] != item)
			current_window[winVarKey_icon]._myDoubles[this._myWorkspace.index()] = item;
		else {
			current_window[winVarKey_icon] = item;
			if(current_window.has_focus())
				this._wsBox._currentWindowFocus = item;
		}
	},
	activateIcons: function(movesUp, yPos) {
		this._isListedTo = yPos || this._dock._DATA_icons_ListedTo_default;
		
		let i = this._windows.length,
			showLabel = this._showLabel,
			win;
		
		while(i--) {
			win = this._windows[i];
			win.createIcon(this._iconSize, this._buttonWidth, movesUp, yPos);
			if(showLabel)
				win.addLabel();
		}
	},
	deleteIcon: function(i) {
		let win = this._windows[i],
			realwin = win._myWindow,
			dock = this._dock;
		
		if(this._wsBox._currentWindowFocus == win)
			this._wsBox._currentWindowFocus = false;
		
		win._destroy();
		if(!realwin[dock._winVarKey_addedToAll])
			delete realwin[dock._winVarKey_icon];
		
		
		this._windows.splice(i, 1);
		
		this._updateIconSize();
	},
	removeAllIcons: function(movesUp, noAnimation) {
		if(this._isListedTo)
			this._isListedTo = 0;
		
		let i = this._windows.length,
			win;
		
		while(i--) {
			win = this._windows[i];
			this._checkFor_allWSWindow(win);
			
			win.deleteIcon(movesUp, true, noAnimation);
		}
	},
	moveIcons: function(y) {
		this._isListedTo = y;
		
		let i = this._windows.length;
		
		while(i--) {
			this._windows[i].setY(y);
		}
	},

	zoomIcons: function(pos) {
		let iconSize = this._iconSize,
			buttonWidth = this._buttonWidth,
			isListedTo = this._isListedTo,
			wins = this._windows,
			i = wins.length;
		pos -= buttonWidth/2;
		while(i--) {
			wins[i].zoomTo(iconSize, buttonWidth, pos, isListedTo);
		}
	},
	undoIconsZoom: function() {
		let iconSize = this._iconSize,
			buttonWidth = this._buttonWidth,
			isListedTo = this._isListedTo,
			wins = this._windows,
			i = wins.length;
		
		while(i--) {
			wins[i].undoZoom(iconSize, buttonWidth, isListedTo);
		}
	},
	
	reloadAllLabels: function(showLabel) {//for conf-update
		this._showLabel = showLabel;
		
		let i = this._windows.length,
			win;
		while(i--) {
			win = this._windows[i];
			win.destroyLabel();
			if(showLabel)
				win.addLabel();
				
		}
	},
	_setIconSize: function(l) {//for conf-update
		let dock = this._dock,
			s = (l <= dock._DATA_max_icons) ? dock._CONF_iconSize : Math.floor(dock._DATA_inner_width / l),
			i = l;
			
			this._iconSize = s;
			this._buttonWidth = s;
			
			while(i--) {
				this._windows[i]._changeSize(s, s);
			}
			
			if(!this._wsBox._WSareListed)
				this._updateHeight(s);
			else if(dock._CONF_dockY_type == BOTTOM && dock._box._currentWS == this._myWorkspace.index())
				this.moveIcons(this._wsBox._calcListedBottomPos(this._myWorkspace.index()));
	},
	_setButtonSize: function(l) {//for conf-update
		let dock = this._dock,
			w = (l <= dock._DATA_max_buttons) ? dock._CONF_buttonMaxWidth : Math.floor(dock._DATA_inner_width / l),
			i = l,
			iconSize = this._iconSize,
			noLabel = (w < dock._DATA_iconLabel_minWidth);
		
		this._showLabel = !noLabel;
		this._buttonWidth = w;
		while(i--) {
			this._windows[i]._changeSize(w, iconSize, true, noLabel);
		}
	},
	_updateIconSize: function() {
		let l = this._windows.length,
			dock = this._dock;
		
		if(dock._CONF_expandBox) {
			if(dock._box._currentWS == this._myWorkspace.index())
				dock.setExpandedDockWidth(l, this);
		}
		else if(l >= dock._DATA_max_icons)
			this._setIconSize(l);
		
		else if(dock._CONF_showWindowTitle && l >= dock._DATA_max_buttons)
			this._setButtonSize(l);
	},
	_updateHeight: function(s) {
		s = s || this._iconSize;
		let dock = this._dock;
		
		if(dock._CONF_dockY_type == TOP)
			dock._hideTo = ((s<dock._CONF_dontHideSize) ? dock._dockY : dock._CONF_dontHideSize - s);
		else {
			dock._dockY = dock._DATA_monitorHeight - s - DOCK_PADDING_BOTTOM;
			if(s<dock._CONF_dontHideSize)
				dock._hideTo = dock._dockY;
		}
		
		dock.setDockHeight(s,  (dock._isHidden ? dock._hideTo : false));
	},
	
	_destroy: function() {
		let i = this._windows.length;
		while(i--) {
			this._windows[i]._destroy();
		}
		this._removeListener();
	}
}


function windowIconButton(dock, app, metaWindow, windowClass, i, iconSize, buttonWidth, ypos) {
	this._init(dock, app, metaWindow, windowClass, i, iconSize, buttonWidth, ypos);
}
windowIconButton.prototype = {
	_init: function(dock, app, window, windowClass, i, iconSize, buttonWidth, ypos) {
		this._dock = dock;
		this._myWindow = window;
		this._myWindowClass = windowClass;
		this._myApp = app;
		this._myIndex = i;
		this._myDoubles = []; //link icons to other workspaces for same window
		this._isMinimized = false;
		this._label = false;
		this._labelIsAdded = false;
		
		this._menuManager = new PopupMenu.PopupMenuManager(this);
		
		if(iconSize)//change of iconSize can only be calculated in outer Class, so remembering wouldnt make sense here
			this.createIcon(iconSize, buttonWidth, false, ypos);
		else
			this.actor = false;
	},
	_calcX: function(w) {
		return this._dock._DATA_space_left + this._myIndex*w;
	},
	
	createIcon: function(iconSize, buttonWidth, movesUp, newY) {
		if(this.actor)
			this.deleteIcon(!movesUp, true);
		newY = newY || this._dock._DATA_iconSpacingTop;
		
		let realIconSize = iconSize - ICON_PADDING_SUM,
			item = new St.Button({ style_class: "windowIconButton",
//				button_mask: St.ButtonMask.ONE | St.ButtonMask.TWO, //blind copy...
				reactive: true,
				can_focus: false,
				width: buttonWidth,
				height: iconSize,
				x: this._calcX(buttonWidth),
				y: newY + (movesUp ? -iconSize : iconSize),
				opacity: 0,
				x_fill: true,
				y_fill: false
			}),
			
			group = new St.BoxLayout(),
			icon = this._myApp.create_icon_texture(this._dock._DATA_iconTextureSize),
			dragable = DND.makeDraggable(item);
		
		item._delegate = this;
		dragable.connect("drag-begin", Lang.bind(this, this._startDrag));
		dragable.connect("drag-cancelled", Lang.bind(this, this._endDrag));
		dragable.connect("drag-end", Lang.bind(this, this._endDrag));
		
		this._ID_buttonPressEvent = item.connect("button-release-event", Lang.bind(this, this._onMouseClick));
		
		icon.set_size(realIconSize, realIconSize);
		group.add(icon);
		item.set_child(group);
		
		//item.set_tooltip_text(this._myWindow.title); //strange behaviour and warnings....
		
		
		this._dock.actor.add(item);
		
		Tweener.addTween(item, {
				opacity: 255,
				y: newY,
				time: 0.5,
				transition: "easeOutExpo"
			});
		
		this.actor = item;
		this._icon = icon;
		this._draggable = dragable;
		
		this._showWindowStats();
	},
	deleteIcon: function(movesUp, dontMoveOthers, noAnimation) {
		if(!dontMoveOthers)
			this._moveOthers();
		
		let item = this.actor;
		
		if(!item)
			return;
		this.removeLabel();
		this.actor = false;
		
		item.disconnect(this._ID_buttonPressEvent);
		
		if(!noAnimation) {
			Tweener.addTween(item, {
					opacity: 0,
					y: item.y + (movesUp ? -item.height : item.height),
					time: 0.6,
					transition: "easeOutQuart",
					onComplete: function() {
						this.destroy();
					}
				});
		}
		else
			item.destroy();
	},
	setY: function(y) {
		if(this.actor)
			Tweener.addTween(this.actor, {
				y: y,
				time: this._dock._CONF_showTime,
				transition: "easeOutQuad"
			});
	},
	
	createLabel: function() {
		let winC = this._myWindowClass,
			dock = this._dock;
		this._label = new St.Label({
				text: this._myWindow.title,
				style: "font-size:"+dock._DATA_iconLabel_fontSize+"px; padding:"+dock._DATA_iconLabel_y+"px 0 "+dock._DATA_iconLabel_y+"px "+dock._DATA_iconLabel_x+"px;",
				style_class: "ButtonLabel",
				width: winC._buttonWidth - ICON_PADDING_SUM,
				x:0
				//x: dock._DATA_iconLabel_x,
				//y: dock._DATA_iconLabel_y
			});
		this._ID_titleChanged = this._myWindow.connect("notify::title", Lang.bind(this, function() {this._label.text = this._myWindow.title;}));
	},
	addLabel: function() {
		if(!this.actor)
			return;
		else if(!this._label)
			this.createLabel();
		
		this.actor.get_child().add(this._label);
		this._label.lower(this._icon);
		this._labelIsAdded = true;
	},
	removeLabel: function() {
		if(!this.actor || !this._labelIsAdded)
			return;
		this.actor.get_child().remove_actor(this._label);
		this._labelIsAdded = false;
	},
	destroyLabel: function() {
		if(this._label) {
			if(this.actor && this._labelIsAdded)
				this.actor.get_child().remove_actor(this._label);
			this._label.destroy();
			this._label = null;
			
			this._myWindow.disconnect(this._ID_titleChanged);
		}
		this._labelIsAdded = false;
	},
	
	_changeSize: function(w, h, changeLabel, noLabel) {
		if(changeLabel && this._label) {
			if(noLabel)
				this.removeLabel();
			else {
				if(!this._labelIsAdded)
					this.addLabel();
				
				this._label.set_width(w - ICON_PADDING_SUM);
			}
		}
		
		if(!this.actor)
			return;
		
		Tweener.addTween(this.actor, {
				x: this._calcX(w),
				width: w,
				height: h,
				time: 0.3,
				transition: "easeInOutCubic",
			});
		Tweener.addTween(this._icon, {
				width: h - ICON_PADDING_SUM,
				height: h - ICON_PADDING_SUM,
				time: 0.3,
				transition: "easeInOutCubic",
			});
		
	},
	zoomTo: function(iconSize, buttonWidth, x, y) {
		let calcX = this._calcX(buttonWidth),
			wideness = (buttonWidth/iconSize)*1.5, //1.5 > more icons are affected
			diff = Math.round((calcX - x) / wideness),
			posDiff;
		
		if(diff > iconSize/2)
			posDiff = Math.round(iconSize/2);
		else if(diff < -iconSize/2)
			posDiff = Math.round(-iconSize/2);
		else
			posDiff = Math.round(diff);
		
		diff = Math.min(Math.abs(diff), iconSize);
		
		let add = iconSize - diff,
			newWidth_button = buttonWidth + add,
			newHeight_button = iconSize  + add,
			newSize_icon = newHeight_button - ICON_PADDING_SUM;
	
		this.actor.set_x(calcX + posDiff);
		this.actor.set_size(newWidth_button, newHeight_button);
		this._icon.set_size(newSize_icon, newSize_icon);
		if(this._dock._CONF_dockY_type == BOTTOM)
			this.actor.set_y(y-add);
		
		if(this._label) {
			this._label.set_x(newHeight_button-iconSize);
		}
	},
	undoZoom: function(iconSize, buttonWidth, y) {
		if(this._label)
			this._label.set_x(0);
		
		if(!this.actor)
			return;
		
		this._icon.set_size(iconSize - ICON_PADDING_SUM, iconSize - ICON_PADDING_SUM);
		//this._icon.set_y(0);
		
		Tweener.addTween(this.actor, {
				x: this._calcX(buttonWidth),
				y: y,
				width: buttonWidth,
				height: iconSize,
				time: 0.2,
				transition: "easeInOutCubic",
			});
	},
	_onMouseClick: function(actor, event) {
		let button = event.get_button();
		switch(button) {
			case 1://left
				if(this._myWindow.has_focus())
					this.doMinimize();
				else
					this.doMap();
			break;
			case 2://middle
				switch(this._dock._CONF_middleClick) {
					case MINIMIZE_WINDOW:
						this.doMinimize();
					break;
					case NEW_WINDOW:
						this.doNew();
					break;
					case CLOSE_WINDOW:
						this.doClose();
					break;
				}
			break;
			case 3://right
				if(this._dock._menuIsOpened || this._dock._isHidden)
					return;
				this._dock._menuIsOpened = true;
				
				if(this._menu)
					this._menu._destroy();
				this._menu = new IconMenu(this);
				this._menu.popup();
		}
	},
	
	_startDrag: function() {
		if(this._menu)
			this._menu._destroy();
		this.actor.opacity = 60;
		
		this._dragMonitor = {
				dragMotion: Lang.bind(this, this._onDragMotion)
			};
		DND.addDragMonitor(this._dragMonitor);
		this._dragState = true;
	},
	_endDrag: function() {
		if(!this._dragState)
			return;
		this._dragState = false;
		
		DND.removeDragMonitor(this._dragMonitor);
		
		this._dock._dragPointer.opacity = 0;//actor has to stay because of drag-Animation and I cant use hide() either...
		
		if(!this.actor) //happens, when WS is changed
			return;
		this.actor.opacity = 255;
		this._showWindowStats();
	},
	_onDragMotion: function(drag) {
		let _this = this,
			dock = this._dock,
			current_wsI = this._myWindowClass._myWorkspace.index(),
			pointer = dock._dragPointer,
			x = drag.x - dock.actor.x,
			y = drag.y - dock.actor.y,
		
			startY = ((dock._CONF_dockY_type == TOP) ? dock._CONF_iconSize + dock._DATA_wsList_space : 0),
			wsI = dock._box.getWSunderMousePos(y),
			winBox = dock._box._workspaces[wsI],
			winI = Math.floor((x-dock._DATA_space_left) / winBox._buttonWidth),
			winLe = winBox._windows.length;
		
		if(winI < 0)
			winI = 0;
		else if(winI >= winLe)
			winI = (winLe >= dock._DATA_max_buttons || wsI == current_wsI) ? winLe-1 : winLe;
		
		
		if(wsI == current_wsI && winI != this._myIndex) {
			this._dragMonitor.dragDrop = function() {
					_this._swapTo(winI);
					
					return DND.DragDropResult.CONTINUE;
				};
		}
		else if(wsI != current_wsI && !this._myWindow.is_on_all_workspaces()) {
			this._dragMonitor.dragDrop = function() {
					let wins = winBox._windows,
						l = wins.length;
					
					_this._myWindow.change_workspace_by_index(
							wsI,
							false, // don't create workspace
							global.get_current_time()
						);
					
					//to make sure, window already exists and icon is resized
					//FIXME: bad solution...
					if(winI < l)
						this._ID_menuTimeout = Mainloop.timeout_add(400, Lang.bind(this, function() {
								wins[l]._swapTo(winI);
							}));
					
					return DND.DragDropResult.CONTINUE;
				};
			
		}
		else {
			this._dragMonitor.dragDrop = false;
			
			pointer.set_position(this.actor.x, this.actor.y);
			pointer.set_size(this.actor.width, this.actor.height);
			
			return DND.DragMotionResult.NO_DROP;
		}
		
		
		if(this._dragMonitor.dragDrop) {
			let buttonWidth = winBox._buttonWidth,
				px = winI * buttonWidth + dock._DATA_space_left,
				py = winBox._isListedTo;
			
			pointer.set_position(px, py);
			pointer.set_size(buttonWidth, winBox._iconSize);
			
			return DND.DragMotionResult.MOVE_DROP;
		}
		else
			return DND.DragMotionResult.CONTINUE;
	},
	getDragActor: function() {
		let item = new Clutter.Clone({ source: this.actor })
		//let item = this._myApp.create_icon_texture(this._icon.width);
		item.opacity = 180;
		return item;
	},
	getDragActorSource: function() {
		let item = this._dock._dragPointer;
		item.set_position(this.actor.x, this.actor.y);
		item.opacity = 255;
		item.raise_top();
		return item;
	},
	
	_moveOthers: function() {
		let wins = this._myWindowClass._windows,
			i = wins.length;
		while(--i > this._myIndex) { //I guess thats faster than for()... I guess...
			wins[i]._moveLeft();
		}
	},
	_moveLeft: function() {
		--this._myIndex;
		this._movingTween();
	},
	_moveRight: function() {
		++this._myIndex;
		this._movingTween();
	},
	_movingTween: function() {
		if(!this.actor)
			return;
		Tweener.addTween(this.actor, {
				x: this._calcX(this.actor.width),
				time: 0.3,
				transition: "easeInQuart"
			});
	},
	_swapTo: function(toI) {
		//FIXME: this function is cruel...
		let w = this._myWindowClass._windows,
			i = this._myIndex,
			t = w[i];
		
		
		if(toI < i) //left
			while(--i >= toI) {
				w[i]._moveRight();
				w[i+1] = w[i];
			}
		else //right
			while(++i <= toI) {
				w[i]._moveLeft();
				w[i-1] = w[i];
			}
		
		this._myIndex = toI;
		w[toI] = t; //now: i==toI
		
		if(!this.actor)
			return;
		
		Tweener.addTween(this.actor, {
				x: this._calcX(this.actor.width),
				time: 0.4,
				transition: "easeInQuart"
			});
	},
	_swapIcons: function(moveRight) {
		let w = this._myWindowClass._windows,
			mI = this._myIndex,
			i1, i2;
		
		if(moveRight && mI+1 < w.length) {//right
			i1 = mI;
			i2 = mI+1;
		}
		else if(!moveRight && mI) {//left (mI > 0)
			i1 = mI-1;
			i2 = mI;
		}
		else
			return;
		
		
		let t = w[i1];
		w[i1] = w[i2];
		w[i2] = t;
		
		w[i1]._moveLeft();
		w[i2]._moveRight();
	},
	
	doMinimize: function() {
		this._myWindow.minimize(global.get_current_time());
		this.showMinimized();
	},
	doMap: function() {
		let win = this._myWindow;
		if(win.get_workspace() != global.screen.get_active_workspace())
			win.get_workspace().activate(global.get_current_time());
		win.activate(global.get_current_time());
		if(Main.overview.visible)
			Main.overview.hide();
		this.showMaped();
	},
	doNew: function() {
		this._myApp.open_new_window(this._dock._box._currentWS);
	},
	doClose: function() {
		this._myWindow.delete(global.get_current_time());
	},
	
	_showWindowStats: function() {
		if(this._myWindow.has_focus())
			this.showFocused();
		if(this._isMinimized)
			this.showMinimized();
	},
	showFocused: function() {
		if(this.actor)
			this.actor.add_style_pseudo_class("focused");
		
		if(this._myDoubles.length)
			this._myDoubles.forEach(function(d) {
					if(d.actor)
						d.actor.add_style_pseudo_class("focused");
				});
	},
	removeFocus: function() {
		if(this.actor)
			this.actor.remove_style_pseudo_class("focused");
		
		if(this._myDoubles.length)
			this._myDoubles.forEach(function(d) {
					if(d.actor)
						d.actor.remove_style_pseudo_class("focused");
				});
	},
	showMinimized: function() {
		this._isMinimized=true;
		if(this._icon)
			this._icon.opacity = ICON_MINIMIZED_OPACITY;
		
		if(this._myDoubles.length) {
			this._myDoubles.forEach(function(d) {
					d._isMinimized=true;
					d._icon.opacity = ICON_MINIMIZED_OPACITY;
				});
		}
	},
	showMaped: function() {
		this._isMinimized=false;
		if(this._icon)
			this._icon.opacity = 255;
		
		
		if(this._myDoubles.length)
			this._myDoubles.forEach(function(d) {
					d._isMinimized=false;
					if(d._icon)
						d._icon.opacity = 255;
				});
	},
	
	_destroy: function() {
		this.destroyLabel();
		this.deleteIcon();
	}
};

function DockletMenu(source) {
	this._init(source);
}
DockletMenu.prototype = {
	__proto__: AppDisplay.AppIconMenu.prototype,
	_init: function(source) {
		this._source = source;
		PopupMenu.PopupMenu.prototype._init.call(this, source.actor, St.Align.MIDDLE, ((source._CONF_dockY_type==TOP) ? St.Side.TOP : St.Side.BOTTOM), 0);
		
		source.stopZooming(); //will just stop the Effect, not undo it
		
		this.actor.add_style_class_name("panelDocklet_popup");

		this._ID_activate = this.connect("activate", Lang.bind(this, this._onActivate));
		
		this._ID_openStateChanged = this.connect("open-state-changed", Lang.bind(this, function (menu, isPoppedUp) {
			if (!isPoppedUp) {
				this._source._menuIsOpened = false;
				
				if(this._source._CONF_autohide)
					this._source._hideDock();
				else if(this._source._CONF_zoomEffect)
					this._source.stopZooming();
				
			}
		}));
		
		LayoutManager.addChrome(this.actor);
		
		source._menuManager.addMenu(this, true);
	},
	_redisplay: function() {
		
		this.removeAll();
		this._settingsItem = this._appendMenuItem(_("Settings"));
		this._appendSeparator();
		
		let launchers = global.settings.get_strv(AppFavorites.getAppFavorites().FAVORITE_APPS_KEY),
			i = launchers.length,
			item, icon;
		while(i--) {
			let app = Shell.AppSystem.get_default().lookup_app(launchers[i]);

			if(!app)
				continue;
				
			item = new PopupMenu.PopupMenuItem(app.get_name());
			item._app = app
			this.addMenuItem(item);
			
			icon = app.create_icon_texture(24);
			item.addActor(icon, {align: St.Align.END});
		}
	},
	_onActivate: function(actor, child) {
		if(child == this._settingsItem)
			new SettingsDialog(this._source, this._source._monitorId);
		else if(child._app) {
			let current_ws = this._source._box._currentWS;
			child._app.open_new_window(current_ws);
		}
	},
	_destroy: function() {
		if(this.isOpen)
			this._source._menuIsOpened = false;
		
		this.disconnect(this._ID_activate);
		this.disconnect(this._ID_openStateChanged);
		this.destroy();
	}
}

function IconMenu(source) {
	this._init(source);
}
IconMenu.prototype = {
	__proto__: AppDisplay.AppIconMenu.prototype,

	_init: function(source) {
		this._source = source;
		this._dock = source._dock;
		PopupMenu.PopupMenu.prototype._init.call(this, source.actor, St.Align.MIDDLE, ((this._dock._CONF_dockY_type==TOP) ? St.Side.TOP : St.Side.BOTTOM), 0);
		
		this._dock.stopZooming(); //will just stop the Effect, not undo it
		
		this.actor.add_style_class_name("panelDocklet_popup");

		this._ID_activate = this.connect("activate", Lang.bind(this, this._onActivate));

		this._ID_destroy = source.actor.connect("destroy", Lang.bind(this, this._destroy));
		
		this._ID_openStateChanged = this.connect("open-state-changed", Lang.bind(this, function (menu, isPoppedUp) {
			if (!isPoppedUp) {
				this._dock._menuIsOpened = false;
				if(this._dock._CONF_autohide)
					this._dock._hideDock();
				else if(this._dock._CONF_zoomEffect)
					this._dock.stopZooming();
			}
		}));
		
		LayoutManager.addChrome(this.actor);
		
		source._menuManager.addMenu(this, true);
	},
	_removeListener: function() {
		if(this._ID_clickedLeft && this._Item_moveLeft)
			this._Item_moveLeft.disconnect(this._ID_clickedLeft);
		if(this._ID_clickedRight && this._Item_moveRight)
			this._Item_moveRight.disconnect(this._ID_clickedRight);
		
		if(this._ID_clickedUp && this._Item_moveUp)
			this._Item_moveUp.disconnect(this._ID_clickedUp);
		if(this._ID_clickedDown && this._Item_moveDown)
			this._Item_moveDown.disconnect(this._ID_clickedDown);
		
		if(this._ID_openStateChanged)
			this.disconnect(this._ID_openStateChanged);
		if(this._ID_activate)
			this.disconnect(this._ID_activate);
		if(this._ID_destroy && this._source.actor)
			this._source.actor.disconnect(this._ID_destroy);
		
		if(this._ID_quitApp && this._item_quitApp)
			this._item_quitApp.disconnect(this._ID_quitApp);
		if(this._ID_toggleFav && this._Item_toggleFav)
			this._Item_toggleFav.disconnect(this._ID_toggleFav);
			
		this._ID_clickedLeft = false;
		this._ID_clickedRight = false;
		this._ID_clickedUp = false;
		this._ID_clickedDown = false;
		this._ID_openStateChanged = false;
		this._ID_activate = false;
		this._ID_destroy = false;
		this._ID_quitApp = false;
		this._ID_toggleFav = false;
	},
	_redisplay: function() {
		let source = this._source,
			isBottom = source._dock._CONF_dockY_type == BOTTOM,
			section = new St.BoxLayout({vertical: false}),
			index = source._myIndex;
		
		//windows
		if(isBottom) {
			this._listOtherWindows();
			this._appendSeparator();
		}
		
		//left
		if(index) {//index > 0
			this._item_moveLeft = new St.Button({ style_class: "swapWindowIcon left" });
			section.add(this._item_moveLeft, {expand: false});
			this._ID_clickedLeft = this._item_moveLeft.connect("clicked", Lang.bind(this, this._onActivate));
		}
		//right
		if(index+1 < source._myWindowClass._windows.length) {
			this._item_moveRight = new St.Button({ style_class: "swapWindowIcon right" });
			section.add(this._item_moveRight, {expand: false, x_align: St.Align.MIDDLE});
			this._ID_clickedRight = this._item_moveRight.connect("clicked", Lang.bind(this, this._onActivate));
		}
		
		//middle
		let label = new St.Label( {style_class: "labelLine"});
		section.add(label, { expand: true, x_fill: true, x_align: St.Align.MIDDLE });
		
		if(!this._source._myWindow.is_on_all_workspaces()) {
			//up
			if(this._source._myWindowClass._myWorkspace.index()) {//index > 0
				this._item_moveUp = new St.Button({ style_class: "swapWindowIcon up" });
				section.add(this._item_moveUp, {expand: false});
				this._ID_clickedUp = this._item_moveUp.connect("clicked", Lang.bind(this, this._onActivate));
			}
			//down
			this._item_moveDown = new St.Button({ style_class: "swapWindowIcon down" });
			section.add(this._item_moveDown, {expand: false});
			this._ID_clickedDown = this._item_moveDown.connect("clicked", Lang.bind(this, this._onActivate));
		}
		
		this.addActor(section);
		
		
		//close
		this._item_closeWin = new PopupMenu.PopupMenuItem(_("Close Window"), {style_class: "specialOne"});
		this.addMenuItem(this._item_closeWin);
		
		//map/ minimized
		if(source._isMinimized) {
			this._item_mapWin = new PopupMenu.PopupMenuItem(_("Restore"), {style_class: "specialOne"});
			this.addMenuItem(this._item_mapWin);
		}
		else {
			this._item_minimizeWin = new PopupMenu.PopupMenuItem(_("Minimize"), {style_class: "specialOne"});
			this.addMenuItem(this._item_minimizeWin);
		}
		
		//new
		this._item_newWin = new PopupMenu.PopupMenuItem(_("New Window"), {style_class: "specialOne"});
		this.addMenuItem(this._item_newWin);
		
		
		let subsection = new PopupMenu.PopupSubMenuMenuItem(_("Application"));
		
		this._Item_toggleFav = new PopupMenu.PopupMenuItem((AppFavorites.getAppFavorites().isFavorite(this._source._myApp.get_id()) ? 
				_("Remove from Favorites") :
				_("Add to Favorites")
			), {style_class: "specialOne"});
		subsection.menu.addMenuItem(this._Item_toggleFav);
		
		this._item_quitApp = new PopupMenu.PopupMenuItem(_("Quit Application"), {style_class: "specialOne"});
		subsection.menu.addMenuItem(this._item_quitApp);
		
		this.addMenuItem(subsection);
		
		this._ID_quitApp = this._item_quitApp.connect("activate", Lang.bind(this, this._onActivate));
		this._ID_toggleFav = this._Item_toggleFav.connect("activate", Lang.bind(this, this._onActivate));
		
		//windows
		if(!isBottom) {
			this._appendSeparator();
			this._listOtherWindows();
		}
	},
	_listOtherWindows: function() {
		let source = this._source,
			windows = source._myApp.get_windows();
		if (windows.length) {
			
			//Asuming: fewer windows -> effectiveness of counting sort
			
			let winVarKey_icon = this._dock._winVarKey_icon,
				wsI = -1, //will be 0 in first round
				currentWS = null,
				sortA = [],
				i=windows.length,
				win, t, j, item;
			
			while(i--) {
				win = windows[i];
				if(win.get_workspace() != currentWS) {
					currentWS = win.get_workspace();
					sortA[++wsI] = [];
				}
				
				sortA[wsI][win[winVarKey_icon]._myIndex] = win;
			}
			
			i=sortA.length;
			while(i--) {
				this._appendSeparator();
				t = sortA[i];
				j = t.length;
				while(j--) {
					if(!t[j])
						continue;
					
					win = t[j];
					if(win.has_focus()) {
						item = new St.Label({ text: win.title, style_class: "focusedOne"});
						this.addActor(item);
					}
					else if(source._myWindow == win) {
						item = new PopupMenu.PopupMenuItem(win.title, {style_class: "chosenOne"});
						item._myWindow = win;
						this.addMenuItem(item);
					}
					else {
						item = new PopupMenu.PopupMenuItem(win.title, {style_class: "otherOne"});
						item._myWindow = win;
						this.addMenuItem(item);
					}
				}
			}
		}
	},
	
	_onActivate: function (actor, child) {
		if (child._myWindow) {
			let win = child._myWindow;
			if(win.get_workspace() != global.screen.get_active_workspace())
				win.get_workspace().activate(global.get_current_time());
			win.activate(global.get_current_time());
		}
		else if (child == this._item_newWin)
			this._source.doNew();
		
		else if (child == this._item_mapWin)
			this._source.doMap();
		
		else if (child == this._item_minimizeWin)
			this._source.doMinimize();
		
		else if (child == this._item_closeWin)
			this._source.doClose();
		else if (actor == this._item_moveRight)
			this._source._swapIcons(true);
		
		else if (actor == this._item_moveLeft)
			this._source._swapIcons(false);
		
		else if(actor == this._item_moveUp && this._source._myWindowClass._myWorkspace.index()) {
			this.close();
			this._source._myWindow.change_workspace_by_index(
					this._source._myWindowClass._myWorkspace.index()-1,
					false, // don't create workspace
					global.get_current_time()
				);
			return;
		}
		else if(actor == this._item_moveDown) {
			this.close();
			this._source._myWindow.change_workspace_by_index(
					this._source._myWindowClass._myWorkspace.index()+1,
					false, // don't create workspace
					global.get_current_time()
				);
			return;
		}
		else if (actor == this._item_quitApp)
			this._source._myApp.request_quit();
		
		else if (actor == this._Item_toggleFav) {
			let favs = AppFavorites.getAppFavorites(),
				favId = this._source._myApp.get_id(),
				isFav = favs.isFavorite(favId);
			if (isFav)
				favs.removeFavorite(favId);
			else
				favs.addFavorite(favId);
		}
		this.close();
		this._removeListener();
	},
	
	_destroy: function() {
		if(this.isOpen)
			this._dock._menuIsOpened = false;
		this._removeListener();
		this.destroy();
	}
}

function SettingsDialog(dock, monitorId, showWelcome) {
	this._init(dock, monitorId, showWelcome);
}
SettingsDialog.prototype = {
	__proto__: ModalDialog.ModalDialog.prototype,
	_init: function(dock, monitorId, showWelcome) {
		this._dock = dock;
		this._isPrimary = (monitorId=="primary");
		
		ModalDialog.ModalDialog.prototype._init.call(this, { styleClass: "panelDocklet_dialog" });
		
		let mainBox = new St.BoxLayout({style_class: "polkit-dialog-main-layout", vertical: true});
			this.contentLayout.add(mainBox, {x_fill: true, y_fill: true});
		
		
		let header = new St.BoxLayout({vertical: false});
			mainBox.add(header, {y_align: St.Align.START, y_fill: false, x_fill: true});
		
		if(this._isPrimary) {
			this._isPrimaryLabel = new St.Label({style_class: "polkit-dialog-headline", style:"color:red;", text:_("Main-Box - ")});
				header.add(this._isPrimaryLabel, {y_fill: false});
		}
		this._headline = new St.Label({style_class: "polkit-dialog-headline"});
			header.add(this._headline, {y_fill: false});
		
		this._description = new St.Label({style_class: "polkit-dialog-description"});
			mainBox.add(this._description, {y_fill: true});
	
		this._messageBox = new St.BoxLayout({style_class: "polkit-dialog-message-layout", vertical: true});
			mainBox.add(this._messageBox, {y_align: St.Align.START, y_fill: true, x_fill: true});
			
		if(!showWelcome)
			this._preconfigurations();
		else
			this._welcome();
		
		this.open();
	},
	
	_createSwitch: function(dock, group, switched, settingsUrl, desc, fu) {
		let section = new St.BoxLayout({vertical: false, style:"padding: 5px"}),
			label = new St.Label({style_class: "polkit-dialog-description", text: desc}),
			button = new St.Button({reactive: true, can_focus: false}),
			switchObj = new PopupMenu.Switch(switched);
			
		section.add(label, {expand:true});
		button.set_child(switchObj.actor);
		section.add(button);
		group.addActor(section, {x_fill: true});
		
		button.connect("button-press-event",  Lang.bind(switchObj, fu || function() {
				this.toggle();
				dock.settings.set_boolean(settingsUrl, this.state);
			}));
	},
	_createCombo: function(dock, group, value, settingsUrl, items, desc, fu) {
		let section = new St.BoxLayout({vertical: false, style:"padding: 5px"}),
			label = new St.Label({style_class: "polkit-dialog-description", text: desc}),
			combo = new PopupMenu.PopupComboBoxMenuItem({ style_class: "panelDocklet_combo" });
			
		section.add(label, {expand:true});
		section.add(combo.actor);
		group.addActor(section, {x_fill: true});
		
		items.forEach(function(o) {
				let item = new PopupMenu.PopupMenuItem(_(o[0]));
				combo.addMenuItem(item, o[1]);
			});
		combo.setActiveItem(value);
		combo.connect("active-item-changed", Lang.bind(this, fu || function(menuItem, id) {
				dock.settings.set_enum(settingsUrl, id);
			}));	
	},
	_createText: function(dock, group, value, settingsUrl, desc, fu) {
		let section = new St.BoxLayout({vertical: false, style:"padding:5px"}),
			label = new St.Label({style_class: "polkit-dialog-description", text: desc}),
			entry = new St.Entry({style_class: "panelDocklet_preferences_text", text: value.toString()});
		
		section.add(label, {expand: true});
		section.add(entry, {expand: false});
		group.addActor(section);
		
		entry.clutter_text.connect("key-focus-out", Lang.bind(this, fu || function(o) { 
				let i = o.get_text();
				o.set_text(i);
				dock.settings.set_int(settingsUrl, parseInt(i));
			}));
		
		return section;
	},
	_createSlider: function(dock, group, value, settingsUrl, desc, fu) {
		let section = new St.BoxLayout({vertical: false, style:"padding: 5px"}),
			label = new St.Label({style_class: "polkit-dialog-description", text: desc}),
			slider = new PopupMenu.PopupSliderMenuItem(value);
		
		section.add(label, {expand:true});
		section.add(slider.actor);
		group.addActor(section, {x_fill: true});
		
		slider.connect("drag-end",  Lang.bind(slider, fu || function() {
				dock.settings.set_double(settingsUrl, this._value);
			}));
		
	},
	_createPreconfiguration: function(group, styleName, title, fu) {
		let button = new St.Button({reactive: true, can_focus: false, style_class: "screenshot"}),
			section = new St.BoxLayout({vertical: true}),
			lable = new St.Label({style_class: "polkit-dialog-headline", text: title}),
			pic = new St.Bin({style_class: styleName});
		
		button.set_child(section);
		section.add(lable, {expand:true});
		section.add(pic);
		group.addActor(button);
		
		button.connect("button-press-event",  Lang.bind(this, fu));
	},
	
	_welcome: function() {
		this._isPrimaryLabel.hide();
		this._headline.text = _("Greetings, stranger!");
		this._description.text = _("This seems to be the first time, you have enabled the panel docklet. \n\
There are a lot of settings, that can help you to costumize this extention to your needs. \n\
Also, there are four preconfigurations to show you, in which different ways you can change the docklet. \n\
\n\
You can skip this dialog and just use the standard-settings or come back at any time \n\
by clicking with the right mouse-button on the docklet  (not on an icon) and choosing \"Settings\".\n\n\
\n\
Please feel free to contact me if you find bug, have suggestions, critics or feedback.\n\
It will help, keeping my coding sprit raise up :) \n\
\n\
Have fun!\n\
Jodli (pw3@gmx.at)");
	
		let dockS = this._dock.settings,
			w = this._dock._DATA_monitorWidth,
			xLeft = 250,
			xRightFromMiddle = 80,
			width = (w/2 - xRightFromMiddle)-xLeft;
		dockS.set_double("dock-x-percent", 1/(w/(xLeft+width)), true);
		dockS.set_double("dock-width-percent", 1/(w/width), true);
		this._dock._update();
	
		this.setButtons([
				{
					label: _("Close"),
					action: Lang.bind(this, function() {
						this.close();
					}),
					key: Clutter.Escape
				},
				{
					label: _("Next"),
					action: Lang.bind(this, function() {
						this._isPrimaryLabel.show();
						this._preconfigurations();
					})
				}
			]);
	
	},
	_preconfigurations: function() {
		this._messageBox.destroy_children(); //FIXME: some objects create NULL-pointer errors in console...
		
		this._headline.text = _("Preconfigurations (1/4)");
		this._description.text = _("You can chose one of already existing preconfigurations. \n\
But this are just variations to show you some possibilities. \n\
You can mix all this settings as you prefer.");
		
		let group = new PopupMenu.PopupComboMenu();
		this._messageBox.add(group.actor);
		
		//panel-docklet
		this._createPreconfiguration(group, "panelDocklet", _("Panel-Docklet"), function() {
				let dockS = this._dock.settings,
					w = this._dock._DATA_monitorWidth,
					xLeft = 250,
					xRightFromMiddle = 80,
					width = (w/2 - xRightFromMiddle)-xLeft;
				
				dockS.set_boolean("autohide", true, true);
				dockS.set_boolean("zoom-effect", false, true);
				dockS.set_boolean("list-all-workspaces", false, true);
				dockS.set_boolean("show-window-title", false, true);
				dockS.set_boolean("show-workspace-number", true, true);
				dockS.set_boolean("cut-view", false, true);
				dockS.set_boolean("expand-box", false, true);
				dockS.set_enum("dock-y", 0, true);
				
				dockS.set_enum("fix-docklet-position-at", 2, true);
				dockS.set_double("dock-x-percent", 1/(w/(xLeft+width)), true);
				dockS.set_double("dock-width-percent", 1/(w/width), true);
				dockS.set_int("dont-hide-size", 18, true);
				dockS.set_int("basic-icon-size", 25, true);
				dockS.set_int("basic-button-width", width, true);
				
				this._dock._reload();
				this._features();
			});
		group.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
		
		//panel
		this._createPreconfiguration(group, "panel", _("Panel"), function() {
				let dockS = this._dock.settings,
					w = this._dock._DATA_monitorWidth;
				
				dockS.set_boolean("autohide", false, true);
				dockS.set_boolean("zoom-effect", false, true);
				dockS.set_boolean("list-all-workspaces", false, true);
				dockS.set_boolean("show-window-title", true, true);
				dockS.set_boolean("show-workspace-number", true, true);
				dockS.set_boolean("cut-view", true, true);
				dockS.set_boolean("expand-box", false, true);
				dockS.set_enum("dock-y", 1, true);
				
				dockS.set_enum("fix-docklet-position-at", 1, true);
				dockS.set_double("dock-x-percent", 0.5, true);
				dockS.set_double("dock-width-percent", 1, true);
				dockS.set_int("basic-icon-size", 25, true);
				dockS.set_int("basic-button-width", w/5, true);
				
				this._dock._reload();
				this._features();
			});
		group.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
		
		//big panel
		this._createPreconfiguration(group, "big_panel", _("Big Panel"), function() {
				let dockS = this._dock.settings,
					w = this._dock._DATA_monitorWidth;
				
				dockS.set_boolean("autohide", true, true);
				dockS.set_boolean("zoom-effect", false, true);
				dockS.set_boolean("list-all-workspaces", true, true);
				dockS.set_boolean("show-window-title", true, true);
				dockS.set_boolean("show-workspace-number", true, true);
				dockS.set_boolean("cut-view", false, true);
				dockS.set_boolean("expand-box", false, true);
				dockS.set_enum("dock-y", 0, true);
				
				dockS.set_enum("fix-docklet-position-at", 1, true);
				dockS.set_double("dock-x-percent", 0.5, true);
				dockS.set_double("dock-width-percent", 1, true);
				dockS.set_int("basic-icon-size", 30, true);
				dockS.set_int("dont-hide-size", 0, true);
				dockS.set_int("basic-button-width", w/5, true);
				
				this._dock._reload();
				this._features();
			});
		group.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
		
		//docklet
		this._createPreconfiguration(group, "docklet", _("Docklet"), function() {
				let dockS = this._dock.settings;
				
				dockS.set_boolean("autohide", true, true);
				dockS.set_boolean("zoom-effect", true, true);
				dockS.set_boolean("list-all-workspaces", false, true);
				dockS.set_boolean("show-workspace-number", false, true);
				dockS.set_boolean("show-window-title", false, true);
				dockS.set_boolean("cut-view", false, true);
				dockS.set_boolean("expand-box", true, true);
				dockS.set_enum("dock-y", 1, true);
				
				dockS.set_enum("fix-docklet-position-at", 1, true);
				dockS.set_double("dock-x-percent", 0.5, true);
				dockS.set_double("dock-width-percent", 0.1, true);
				dockS.set_int("dont-hide-size", 0, true);
				dockS.set_int("basic-icon-size", 45, true);
				
				this._dock._reload();
				this._features();
			});
		group.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
		
		let defaultItem = new PopupMenu.PopupMenuItem(_("Restore Defaults"));
		defaultItem.connect("activate", Lang.bind(this, function() {
				this._dock.settings.restoreDefault(Extention_path);
				this._dock._reload();
			}));
		group.addMenuItem(defaultItem);
		
		this.setButtons([
				{
					label: _("Close"),
					action: Lang.bind(this, function() {
						this.close();
					}),
					key: Clutter.Escape
				},
				{
					label: _("Next (Features)"),
					action: Lang.bind(this, function() {
						this._features();
					})
				}
			]);
	},
	_features: function() {
		this._messageBox.destroy_children(); //FIXME: some objects create NULL-pointer errors in console...
		
		let dock = this._dock,
			group = new PopupMenu.PopupComboMenu(),
			_this = this;
		this._messageBox.add(group.actor);
		
		this._headline.text = _("Features (2/4)");
		this._description.text = _("This settings are affecting the behaviour, appearance and speed of the extention.");
		
		if(this._isPrimary)
			this._createSwitch(dock, group, dock._CONF_onAllScreens, "on-all-screens",
				_("On all screens:\n\t(a box is created on every screen, which is managed seperatly)"));
		
		this._createSwitch(dock, group, dock._CONF_onlyScreenWindows, "only-screen-windows",
			_("Ignore other Screens:\n\t(this box will only list icons, which are on its monitor)"));
			
		this._createSwitch(dock, group, dock._CONF_autohide, "autohide",
				_("Auto-hide:"), function() {
					this.toggle();
					dock.settings.set_boolean("autohide", this.state);
					if(this.state)
						_this._Text_dontHideSize.show();
					else
						_this._Text_dontHideSize.hide();
				});
		this._Text_dontHideSize = this._createText(dock, group, dock._CONF_dontHideSize, "dont-hide-size",
				_("Hidden-size:\n\t(how much should stay in the view, when the box is hidden)"), function(o) { 
				let i = parseInt(o.get_text());
				
				if(i<0)
					i=0;
				o.set_text(i.toString());
				dock.settings.set_int("dont-hide-size", i);
			});
		if(!dock._CONF_autohide)
			this._Text_dontHideSize.hide();
		
		this._createCombo(dock, group, dock._CONF_middleClick, "middle-click-action", [[_("New Window"), NEW_WINDOW], [_("Close Window"), CLOSE_WINDOW], [_("Minimize"), MINIMIZE_WINDOW]],
				_("Middle-click-action:\n\t(on Icons)"));
		
		this._createSwitch(dock, group, dock._CONF_zoomEffect, "zoom-effect",
				_("Zoom-effect on mouse-over:"));
		this._createSwitch(dock, group, dock._CONF_showAllWS, "list-all-workspaces",
				_("List other workspaces too:\n\t(if auto-hide is enabled, only on mouse-over)"));
		this._createSwitch(dock, group, dock._CONF_showWSline, "show-workspace-number",
				_("Show workspace-Indicator:"));
		this._createSwitch(dock, group, dock._CONF_showWindowTitle, "show-window-title",
				_("Show window-title next to icon:"), function() {
					this.toggle();
					dock.settings.set_boolean("show-window-title", this.state);
					if(this.state)
						_this._Text_buttonMaxWidth.show();
					else
						_this._Text_buttonMaxWidth.hide();
				});
				
		this._Text_buttonMaxWidth = this._createText(dock, group, dock._CONF_buttonMaxWidth, "basic-button-width",
				_("Maximum width of Buttons:\n(Icon and Label)"));	
		if(!dock._CONF_showWindowTitle)
			this._Text_buttonMaxWidth.hide();
				
		this._createSwitch(dock, group, dock._CONF_cutStruts, "cut-view",
				_("Cut the view-region:\n\t(so the box wouldn't cover parts of the screen)"));
		
		this.setButtons([
				{
					label: _("Back (Preconfigurations)"),
					action: Lang.bind(this, function() {
						this._preconfigurations();
					})
				},
				{
					label: _("Close"),
					action: Lang.bind(this, function() {
						this.close();
					}),
					key: Clutter.Escape
				},
				{
					label: _("Next (Appearance)"),
					action: Lang.bind(this, function() {
						this._appearance();
					})
				}
			]);
	},
	_appearance: function() {
		this._messageBox.destroy_children(); //FIXME: some objects create NULL-pointer errors in console...
		
		let dock = this._dock,
			group = new PopupMenu.PopupComboMenu();
		this._messageBox.add(group.actor);
		
		this._headline.text = _("Appearance (3/4)");
		this._description.text = "";
		
		this._createSwitch(dock, group, dock._CONF_expandBox, "expand-box",
				_("Box-width depends on number of Icons:\n\t(if enabled, the set Box-width will be considered as min-width)"));
		this._createCombo(dock, group, dock._CONF_dockY_type, "dock-y", [[_("Top"), TOP], [_("Bottom"), BOTTOM]],
				_("Position of box:"));
		this._createCombo(dock, group, dock._CONF_fixDockPositionAt, "fix-docklet-position-at", [[_("Left"), LEFT], [_("Middle"), MIDDLE], [_("Right"), RIGHT]],
				_("Position fixes box at:"), function(menuItem, id) {
					if(id == RIGHT && dock._CONF_dockX_percent < 0.1)
						dock.settings.set_double("dock-x-percent", 0.1);
					else if(id == LEFT && dock._CONF_dockX_percent > 0.9)
						dock.settings.set_double("dock-x-percent", 0.9);
					
					dock.settings.set_enum("fix-docklet-position-at", id);
				});
		
		this._createSlider(dock, group, dock._CONF_dockWidth_percent, "dock-width-percent",
				_("Box-width"), function() {
					dock.settings.set_double("dock-width-percent", Math.max(this._value, 0.1));
				});
		this._createSlider(dock, group, dock._CONF_dockX_percent, "dock-x-percent",
				_("Box-x-position"), function() {
					let value = this._value;
					if(dock._CONF_fixDockPositionAt == RIGHT && value < 0.1)
						value = 0.1;
					else if(dock._CONF_fixDockPositionAt == LEFT && value > 0.9)
						value = 0.9;
					dock.settings.set_double("dock-x-percent", value);
				});
		this._createText(dock, group, dock._CONF_iconSize, "basic-icon-size",
				_("Icon-size:"), function(o) { 
				let i = parseInt(o.get_text());
				
				if(i<10)
					i=10;
				o.set_text(i.toString());
				dock.settings.set_int("basic-icon-size", i);
			});
			
		this.setButtons([
				{
					label: _("Back (Features)"),
					action: Lang.bind(this, function() {
						this._features();
					})
				},
				{
					label: _("Close"),
					action: Lang.bind(this, function() {
						this.close();
					}),
					key: Clutter.Escape
				},
				{
					label: _("Next (Fine tuning)"),
					action: Lang.bind(this, function() {
						this._details();
					})
				}
			]);
	},
	_details: function() {
		this._messageBox.destroy_children(); //FIXME: some objects create NULL-pointer errors in console...
		
		let dock = this._dock,
			group = new PopupMenu.PopupComboMenu();
		this._messageBox.add(group.actor);
		
		this._headline.text = _("Fine tuning (4/4)");
		this._description.text = _("This settings are only considered if Auto-hide is enabled.");
		
		this._createText(dock, group, dock._CONF_menuTimeout, "time-until-hide",
				_("Waiting-time until hide-begin:"));
		this._createText(dock, group, dock._CONF_showTime, "show-time",
				_("Show-time:"), function(o) { 
					let i = o.get_text();
					o.set_text(i);
					dock.settings.set_double("show-time", i);
				});
		this._createText(dock, group, dock._CONF_hideTime, "hide-time",
				_("Hide-time:"), function(o) { 
					let i = o.get_text();
					o.set_text(i);
					dock.settings.set_double("hide-time", i);
				});
			
		this.setButtons([
				{
					label: _("Back (Appearance)"),
					action: Lang.bind(this, function() {
						this._appearance();
					})
				},
				{
					label: _("Close"),
					action: Lang.bind(this, function() {
						this.close();
					}),
					key: Clutter.Escape
				}
			]);
	},
}

function addSecondaryDocklets() {
	let primaryIndex = LayoutManager.primaryIndex,
		i = LayoutManager.monitors.length;
	
	while(i--) {
		if(i != primaryIndex)
			secondaryPanelDocklets[i] = new panelDocklet(i, false);
	}
}
function removeSecondaryDocklets() {
	secondaryPanelDocklets.forEach(function(o) {o._destroy()});
	secondaryPanelDocklets = [];
}

function init(metadata) {
	global.log("Be carefull! This year, Christmas falls on a Friday the 13th!");
	
	Gettext.textdomain("panel-docklet@quina.at");
	Gettext.bindtextdomain("panel-docklet@quina.at", Extention_path + "/locale");
}
function enable() {
	mainPanelDocklet = new panelDocklet("primary", secondaryPanelDocklets);
	if(mainPanelDocklet._CONF_onAllScreens)
		addSecondaryDocklets();
	
	//this is not a real connect!
	mainPanelDocklet.settings.connect("on-all-screens", Lang.bind(this, function(k,v) {
			removeSecondaryDocklets();
	
			if((mainPanelDocklet._CONF_onAllScreens = v)) {//Variable is changed here
				addSecondaryDocklets();
				mainPanelDocklet._secondaryPanelDocklets = secondaryPanelDocklets;
			}
			mainPanelDocklet._reload();
		}));
	
	ID_monitorEvent = global.screen.connect("monitors-changed", Lang.bind(this, function() {
			removeSecondaryDocklets();
			addSecondaryDocklets();
			mainPanelDocklet._secondaryPanelDocklets = secondaryPanelDocklets;
			
			mainPanelDocklet._reload();
		}));
	
	
	if(mainPanelDocklet.settings.get_boolean("first-time"))
		Mainloop.timeout_add(2000, Lang.bind(this, function() {
				new SettingsDialog(mainPanelDocklet, "primary", true);
				mainPanelDocklet.settings.set_boolean("first-time", false);
			}));
}
function disable() {
	mainPanelDocklet._destroy();
	mainPanelDocklet = null;
	
	removeSecondaryDocklets();
	
	global.screen.disconnect(ID_monitorEvent);
}
